from typing import Tuple
import re
import os

from colorama import Fore, Style
from tqdm import tqdm
from loguru import logger

from .logging import downloaded_log


def get_format_type(_format: str = "epub") -> int:
    if re.search(r"\bepub\b", _format, re.I):
        format_type = 0

    elif re.search(r"\bmobi\b", _format, re.I):
        format_type = 1

    elif re.search(r"\bpdf\b", _format, re.I):
        format_type = 2

    elif re.search(r"\bhtml\b", _format, re.I):
        format_type = 3

    elif re.search(r"\azw3\b", _format, re.I):
        format_type = 4

    else:  # default epub format
        format_type = 0

    return format_type


def check_url(pbar, url: str, debug: bool = False,
              exit_status: int = 0) -> Tuple[bool, int]:

    if re.search(r"archiveofourown.org", url):
        supported_flag = True

    else:
        supported_flag = False

    if not supported_flag:
        pbar.update(1)
        exit_status = 1

        if debug:
            logger.error(
                f"Skipping unsupported URL: {url}\nOnly archiveofourown.org is supported.")
        else:
            tqdm.write(
                Fore.RED + f"Skipping unsupported URL: {url}" +
                Style.RESET_ALL + "\nOnly archiveofourown.org is supported.")

    return supported_flag, exit_status


def save_data(fic, out_dir: str, file_name:  str, download_url: str,
              debug: bool, force: bool, exit_status: int) -> int:

    ebook_file = out_dir+file_name

    if os.path.exists(out_dir+file_name) and force is False:

        exit_status = 1
        if debug:
            logger.error(
                f"{out_dir+file_name} already exists. Skipping download. Use --force flag to overwrite.")

        else:
            tqdm.write(
                Fore.RED +
                f"{out_dir+file_name} already exists. Skipping download. Use --force flag to overwrite.")

    else:
        if force and debug:
            logger.warning(
                f"--force flag was passed. Overwriting {out_dir+file_name}")

        fic.get_fic_data(download_url)

        downloaded_log(debug, file_name)

        with open(ebook_file, "wb") as f:
            f.write(fic.response_data.content)

        exit_status = 0

    return exit_status


def show_urls_from_page(fic):
    exit_status = 0
    found_flag = False
    if fic.ao3_works_list:
        found_flag = True
        tqdm.write(Fore.GREEN +
                   f"\nFound {len(fic.ao3_works_list)} works urls.")
        ao3_works_list = '\n'.join(fic.ao3_works_list)
        tqdm.write(ao3_works_list)

    if fic.ao3_series_list:
        found_flag = True
        tqdm.write(Fore.GREEN +
                   f"\nFound {len(fic.ao3_series_list)} series urls.")
        ao3_series_list = '\n'.join(fic.ao3_series_list)
        tqdm.write(ao3_series_list)

    if found_flag is False:
        tqdm.write(Fore.RED + "\nFound 0 urls.")
        exit_status = 1

    return exit_status



import re
import requests
from tqdm import tqdm
from colorama import Fore, Style
from loguru import logger
from bs4 import BeautifulSoup
from rich.console import Console

from .ao3 import ArchiveOfOurOwn
from .logging import init_log, download_processing_log
from .processing import check_url, save_data

bar_format = "{l_bar}{bar}| {n_fmt}/{total_fmt}, {rate_fmt}{postfix}, ETA: {remaining}"
console = Console()


class FetchData:
    def __init__(self, format_type="epub", out_dir="", force=False,
                 debug=False):
        self.format_type = format_type
        self.out_dir = out_dir
        self.force = force
        self.debug = debug
        self.exit_status = 0

    def get_fic_with_infile(self, infile: str):

        try:
            with open(infile, "r") as f:
                urls = f.read().splitlines()

        except FileNotFoundError:
            tqdm.write(
                Fore.RED +
                f"{infile} file could not be found. Please enter a valid file path.")
            exit(1)

        init_log(self.debug, self.force)
        if self.debug:
            logger.debug("Calling get_fic_with_infile()")

        url_list = []
        for url in urls:
            if not re.search(r"\barchiveofourown.org/works/\b", url):
                self.get_urls_from_page(url)
                if self.ao3_works_list:
                    url_list += self.ao3_works_list

            else:
                url_list.append(url)

        if not url_list:
            exit(1)

        with tqdm(total=len(url_list), ascii=False,
                  unit="file", bar_format=bar_format) as pbar:

            for url in url_list:

                supported_url, self.exit_status = check_url(
                    pbar, url, self.debug, self.exit_status)
                if supported_url:
                    try:
                        download_processing_log(self.debug, url)
                        fic = ArchiveOfOurOwn(url, self.debug,
                                              self.exit_status)
                        fic.get_fic_metadata(self.format_type)

                        # update the exit status
                        self.exit_status = fic.exit_status

                        if fic.ao3_works_name is None:
                            tqdm.write(
                                Fore.RED +
                                "Fanfiction not found")
                            self.exit_status = 1
                            continue

                        if fic.file_name is None:
                            self.exit_status = 1

                        else:
                            self.exit_status = save_data(
                                fic, self.out_dir,
                                fic.file_name,
                                fic.download_url, self.debug, self.force,
                                self.exit_status)

                        pbar.update(1)

                    # Error: 'ArchiveOfOurOwn' object has no attribute 'file_name'
                    # Reason: Unsupported URL
                    except AttributeError:
                        pbar.update(1)
                        self.exit_status = 1
                        pass  # skip the unsupported url

                else:  # skip the unsupported url
                    continue

    def get_fic_with_list(self, list_url: str):

        urls = list_url.split(",")

        init_log(self.debug, self.force)
        if self.debug:
            logger.debug("Calling get_fic_with_list()")

        url_list = []
        for url in urls:
            if not re.search(r"\barchiveofourown.org/works/\b", url):
                self.get_urls_from_page(url)
                if self.ao3_works_list:
                    url_list += self.ao3_works_list

            else:
                url_list.append(url)

        if not url_list:
            exit(1)

        with tqdm(total=len(url_list), ascii=False,
                  unit="file", bar_format=bar_format) as pbar:

            for url in url_list:

                supported_url,  self.exit_status = check_url(
                    pbar, url, self.debug, self.exit_status)

                if supported_url:
                    try:
                        download_processing_log(self.debug, url)
                        fic = ArchiveOfOurOwn(
                            url, self.debug, self.exit_status)
                        fic.get_fic_metadata(self.format_type)

                        # update the exit status
                        self.exit_status = fic.exit_status

                        if fic.ao3_works_name is None:
                            tqdm.write(
                                Fore.RED +
                                "Fanfiction not found")
                            self.exit_status = 1
                            continue

                        if fic.file_name is None:
                            self.exit_status = 1

                        else:
                            self.exit_status = save_data(
                                fic, self.out_dir,
                                fic.file_name,
                                fic.download_url, self.debug, self.force,
                                self.exit_status)

                        pbar.update(1)

                    # Error: 'ArchiveOfOurOwn' object has no attribute 'file_name'
                    # Reason: Unsupported URL
                    except AttributeError:
                        pbar.update(1)
                        self.exit_status = 1
                        pass  # skip the unsupported url

                else:  # skip the unsupported url
                    continue

    def get_fic_with_url(self, url: str):

        init_log(self.debug, self.force)
        if self.debug:
            logger.debug("Calling get_fic_with_url()")

        url_list = []
        if not re.search(r"\barchiveofourown.org/works/\b", url):
            self.get_urls_from_page(url)
            if self.ao3_works_list:
                url_list += self.ao3_works_list

        else:
            url_list.append(url)

        if not url_list:
            exit(1)

        with tqdm(total=len(url_list), ascii=False,
                  unit="file", bar_format=bar_format) as pbar:

            for url in url_list:

                supported_url, self.exit_status = check_url(
                    pbar, url, self.debug, self.exit_status)

                if supported_url:
                    try:
                        download_processing_log(self.debug, url)

                        fic = ArchiveOfOurOwn(
                            url, self.debug,  self.exit_status)
                        fic.get_fic_metadata(self.format_type)

                        # update the exit status
                        self.exit_status = fic.exit_status

                        if fic.ao3_works_name is None:
                            tqdm.write(
                                Fore.RED +
                                "Fanfiction not found")
                            self.exit_status = 1
                            exit(self.exit_status)

                        if fic.file_name is None:
                            self.exit_status = 1

                        else:
                            self.exit_status = save_data(
                                fic,
                                self.out_dir, fic.file_name,
                                fic.download_url, self.debug, self.force,
                                self.exit_status)

                        pbar.update(1)

                    # Error: 'ArchiveOfOurOwn' object has no attribute 'file_name'
                    # Reason: Unsupported URL
                    except AttributeError:
                        pbar.update(1)
                        self.exit_status = 1
                        pass  # skip the unsupported url

                else:  # skip the unsupported url
                    pass

    def get_urls_from_page(self, url: str):

        with console.status("[bold green]Processing..."):
            response = requests.get(url)

            if self.debug:
                logger.debug(f"GET: {response.status_code}: {response.url}")

            html_page = BeautifulSoup(response.content, 'html.parser')

            if re.search("archiveofourown.org", url):
                ao3_series_works_html = []
                self.ao3_works_list = []
                self.ao3_series_list = []

                ao3_series_works_html_h4 = html_page.findAll(
                    'h4', attrs={'class': 'heading'})

                for i in ao3_series_works_html_h4:
                    ao3_series_works_html.append(i)

                ao3_series_works_html = ""
                for i in ao3_series_works_html_h4:
                    ao3_series_works_html += str(i)

                ao3_urls = BeautifulSoup(ao3_series_works_html, 'html.parser')

                for tag in ao3_urls.findAll('a', {'href': re.compile('/works/')}):
                    self.ao3_works_list.append(
                        "https://archiveofourown.org"+tag['href'])

                for tag in ao3_urls.findAll('a', {'href': re.compile('/series/')}):
                    self.ao3_series_list.append(
                        "https://archiveofourown.org"+tag['href'])

            else:
                self.ao3_works_list = None
                self.ao3_series_list = None
                tqdm.write(
                    Fore.RED + f"\nSkipping unsupported URL: {url}" + Style.RESET_ALL +
                    Fore.BLUE + "\nOnly archiveofourown.org is supported.")



from colorama import Fore
from loguru import logger
from tqdm import tqdm


def init_log(debug: bool, force: bool):
    if debug:
        if force:
            logger.warning(
                "--force flag was passed. Files will be overwritten.")
    else:
        if force:
            tqdm.write(
                Fore.YELLOW +
                "WARNING: --force flag was passed. Files will be overwritten.")


def downloaded_log(debug: bool, file_name: str):
    if debug:
        logger.info(f"Downloaded {file_name}")
    else:
        tqdm.write(Fore.GREEN + f"Downloaded {file_name}")


def download_processing_log(debug: bool, url: str):
    if debug:
        logger.info(f"Processing {url}")
    else:
        tqdm.write(Fore.BLUE + f"\nProcessing {url}")



import requests
import re
import time
from loguru import logger
from bs4 import BeautifulSoup


params = {
    'view_adult': 'true',
    'view_full_work': 'true'
}


class ArchiveOfOurOwn:
    def __init__(self, BaseUrl: str, debug: bool, exit_status: int):
        self.BaseUrl = BaseUrl
        self.debug = debug
        self.exit_status = exit_status
        self.session = requests.Session()

    def get_fic_metadata(self, format_type: int):

        time.sleep(1)
        response = self.session.get(self.BaseUrl, params=params)

        if self.debug:
            logger.debug(f"GET: {response.status_code}: {response.url}")

        ao3_soup = BeautifulSoup(response.content, 'html.parser')

        try:
            self.ao3_works_name = (ao3_soup.find(
                'h2', attrs={'class': 'title heading'}).contents[0]).strip()

        except AttributeError:
            if self.debug:
                logger.error(
                    "ao3_works_name is missing. Fanfiction not found")
            self.ao3_works_name = None
            return

        self.ao3_author_name_list = ao3_soup.find(
            'h3', attrs={'class': 'byline heading'}).find_all('a')

        self.ao3_author_name = []
        for author in self.ao3_author_name_list:
            self.ao3_author_name.append(author.string.strip())
        self.ao3_author_name = " and ".join(self.ao3_author_name)

        # fetch download urls
        try:
            self.ao3_works_download = ao3_soup.find(
                'ul', attrs={'class': 'expandable secondary'}).findAll('a')

        except AttributeError:
            self.log.info(
                "ao3_works_download is missing. Fanfiction not available for download.")
            self.ao3_works_download = None
            return

        for url in self.ao3_works_download:

            if format_type == 0:
                if re.search("epub", url['href'], re.I):
                    self.download_url = f"https://archiveofourown.org{url['href']}"
                self.file_format = "epub"

            elif format_type == 1:
                if re.search("mobi", url['href'], re.I):
                    self.download_url = f"https://archiveofourown.org{url['href']}"
                self.file_format = "mobi"

            elif format_type == 2:
                if re.search("pdf", url['href'], re.I):
                    self.download_url = f"https://archiveofourown.org{url['href']}"
                self.file_format = "pdf"

            elif format_type == 3:
                if re.search("html", url['href'], re.I):
                    self.download_url = f"https://archiveofourown.org{url['href']}"
                self.file_format = "html"

            elif format_type == 4:
                if re.search("azw3", url['href'], re.I):
                    self.download_url = f"https://archiveofourown.org{url['href']}"
                self.file_format = "azw3"

        self.file_name = f"{self.ao3_works_name} by {self.ao3_author_name}.{self.file_format}"

    def get_fic_data(self, download_url: str):

        self.response_data = self.session.get(
            download_url, allow_redirects=True
        )

        if self.debug:
            logger.debug(
                f"GET: {self.response_data.status_code}: {self.response_data.url}")



import click
import sys
from loguru import logger
from datetime import datetime
from colorama import init, Fore

from .utils.processing import get_format_type, show_urls_from_page
from .utils.fetch_data import FetchData

init(autoreset=True)  # colorama init
time = datetime.now().strftime("%H_%M_%S")


# @logger.catch  # for internal debugging
@click.command(no_args_is_help=True)
@click.option('-u', '--url', help='The url of the fanfiction enclosed within quotes ')
@click.option('-i', '--infile', help='Give a filename to read URLs from')
@click.option('-l', '--list-url', 'list_url',  help='Enter a comma separated list of urls to download, enclosed within quotes')
@click.option('-o', '--out-dir', 'out_dir', default="", help='Absolute path to the Output directory for files (default: Current Directory)')
@click.option('-f', '--format', '_format', default="epub", help='Download Format: EPUB (default), AZW3, MOBI, PDF or HTML')
@click.option('--force', default=False, help=' Force overwrite of an existing file', is_flag=True)
@click.option('--get-urls', 'get_urls', default=None, help='Get all story urls found from a page')
@click.option('-d', '--debug', default=False, help='Show the log in the console for debugging', is_flag=True)
@click.option('--log', default=False, help='Save the logfile for debugging', is_flag=True)
@click.option('-v', '--version', default=False, help='Display version & quit.', is_flag=True)
def run_cli(infile: str, url: str, list_url: str, _format: str, get_urls: str,
            out_dir: str, debug: bool, version: bool, log: bool, force: bool):
    """
    A CLI to download from archiveofourown.org using their built-in download option

    To report issues for the CLI, open an issue at https://github.com/arzkar/ao3-cli/issues
    """

    if log:
        debug = True
        click.echo(
            Fore.GREEN + f"Creating ao3_cli_{time}.log in the current directory")
        logger.add(f"ao3_cli_{time}.log")

    format_type = get_format_type(_format)
    if infile:
        fic = FetchData(format_type, out_dir, force,
                        debug)
        fic.get_fic_with_infile(infile)

    elif list_url:
        fic = FetchData(format_type, out_dir, force,
                        debug)
        fic.get_fic_with_list(list_url)

    elif url:
        fic = FetchData(format_type, out_dir, force,
                        debug)
        fic.get_fic_with_url(url)

    elif get_urls:
        fic = FetchData(debug)
        fic.get_urls_from_page(get_urls)
        fic.exit_status = show_urls_from_page(fic)

    if version:
        click.echo("Version: 0.1.3")
        sys.exit(0)

    sys.exit(fic.exit_status)



from setuptools import setup, find_packages

with open("README.md", "r") as f:
    long_description = f.read()

setup(
    name='ao3-cli',
    author='Arbaaz Laskar',
    author_email="arzkar.dev@gmail.com",
    description="A CLI to download from archiveofourown.org",
    long_description=long_description,
    long_description_content_type="text/markdown",
    version='0.1.3',
    license='MIT',
    url="https://github.com/arzkar/ao3-cli/",
    packages=find_packages(include=['ao3_cli', 'ao3_cli.*']),
    include_package_data=True,
    install_requires=[
        'click>=7.1.2',
        'rich>=10.3.0',
        'requests>=2.25.1',
        'loguru>=0.5.3',
        'tqdm>=4.60.0',
        'BeautifulSoup4>=4.9.3',
        'colorama>=0.4.4'
    ],
    entry_points='''
        [console_scripts]
        ao3_cli=ao3_cli.cli:run_cli
    ''',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
